// geometry class
// -Kyoung Shin Park (2010 Spring)
//////////////////////////////////////////////////////////////////////
#ifndef __GEOMETRY_H__
#define __GEOMETRY_H__

#include <GL/glu.h>
#include "node.h"

#define GLU_SPHERE          1
#define GLU_CYLINDER        2
#define GLU_DISK            3
#define GLU_PARTIAL_DISK    4

#define AXIS_X 0
#define AXIS_Y 1
#define AXIS_Z 2

namespace oglclass
{

class geometry : public node
{
public:
	geometry(void);
	virtual ~geometry(void);
	virtual void draw(void);
	void init();

	void makeSphere(GLdouble radius=1.0,GLint slices=8,GLint stacks=8);
	void makeCylinder(GLdouble baseRadius=1.0, GLdouble topRadius=1.0,
					  GLdouble height=1.0, GLint slices=8, GLint stacks=1,
					  int axis=AXIS_Z);
	void makeCone(GLdouble baseRadius=1.0, GLdouble height=1.0,
					  GLint slices=8, GLint stacks=1, int axis=AXIS_Z);
	void makeDisk(GLdouble innerRadius=0.0, GLdouble outerRadius=1.0,
					  GLint slices=8, GLint rings=1, int axis=AXIS_Z);
	void makePartialDisk(GLdouble innerRadius=0.0, GLdouble outerRadius=1.0,
					  GLint slices=8, GLint rings=1,
					  GLdouble startAngle=0.0, GLdouble sweep=180.0,
					  int axis=AXIS_Z);

	void setUseNormals(GLboolean state);
	void setUseTexture(GLboolean state);
	void setDrawStyle(GLenum style);

	GLUquadric * quadric(void) const { return quadric_; }
	int quadricType(void) const { return quadricType_; }
	GLboolean useNormals(void) const { return useNormals_; }
	GLboolean useTexture(void) const { return useTexture_; }
	GLenum drawStyle(void) const { return drawStyle_; }

protected:
	GLUquadric * quadric_;
	int quadricType_;
	GLdouble radius_, topRadius_, innerRadius_, height_;
	GLdouble startAngle_, sweep_;
	GLint slices_, stacks_, rings_;
	GLboolean useNormals_, useTexture_;
	GLenum drawStyle_;
	int axis_;
};

}

#endif
