// light class
// -Kyoung Shin Park (2009 Spring)
//////////////////////////////////////////////////////////////////////
#include "light.h"

namespace oglclass
{

int light::gLightNum_ = 0;

light::light(void)
{
    if (gLightNum_ < GL_MAX_LIGHTS) {
        lightNumber_ = GL_LIGHT0 + gLightNum_;
        gLightNum_++;
    }
    else {
        std::cerr << "Warning: Maximum number of lights exceeded "
            "(maximum is " << GL_MAX_LIGHTS << ")\n";
        lightNumber_ = GL_LIGHT0 + GL_MAX_LIGHTS - 1;
    }

    ambient_[0] = 0.0f; ambient_[1] = 0.0f; ambient_[2] = 0.0f; ambient_[3] = 1.0f;
    diffuse_[0] = 1.0f; diffuse_[1] = 1.0f; diffuse_[2] = 1.0f; diffuse_[3] = 1.0f;
    specular_[0] = 1.0f; specular_[1] = 1.0f; specular_[2] = 1.0f; specular_[3] = 1.0f;
    position_[0] = 0.0f; position_[1] = 0.0f; position_[2] = 1.0f; position_[3] = 0.0f;

    attenuationConstant_ = 1.0f;
    attenuationLinear_ = 0.0f;
    attenuationQuadratic_ = 0.0f;

    spotDirection_[0] = 0.0f; spotDirection_[1] = 0.0f; spotDirection_[2] = -1.0f;
    spotCutoff_ = 180.0f;
    spotExponent_ = 0.0f;
}

void light::enable(void)
{
    glEnable(lightNumber_);
	glLightfv(lightNumber_, GL_POSITION, position_);
	glLightfv(lightNumber_, GL_AMBIENT, ambient_);
	glLightfv(lightNumber_, GL_DIFFUSE, diffuse_);
	glLightfv(lightNumber_, GL_SPECULAR, specular_);
    glLightf(lightNumber_, GL_CONSTANT_ATTENUATION, attenuationConstant_);
    glLightf(lightNumber_, GL_LINEAR_ATTENUATION, attenuationLinear_);
    glLightf(lightNumber_, GL_QUADRATIC_ATTENUATION, attenuationQuadratic_);
    glLightf(lightNumber_, GL_SPOT_CUTOFF, spotCutoff_);

    if (spotCutoff_ < 180) {
        glLightfv(lightNumber_, GL_SPOT_DIRECTION, spotDirection_);
        glLightf(lightNumber_, GL_SPOT_EXPONENT, spotExponent_);
    }
}

void light::disable(void)
{
    glDisable(lightNumber_);
}

void light::setAmbient(vector4& color)
{
    ambient_[0] = color[0];
    ambient_[1] = color[1];
    ambient_[2] = color[2];
    ambient_[3] = color[3];
}

void light::setAmbient(const float * color)
{
    ambient_[0] = color[0];
    ambient_[1] = color[1];
    ambient_[2] = color[2];
    ambient_[3] = color[3];
}

void light::setAmbient(float r, float g, float b, float a)
{
    ambient_[0] = r;
    ambient_[1] = g;
    ambient_[2] = b;
    ambient_[3] = a;
}

void light::setDiffuse(vector4& color)
{
    diffuse_[0] = color[0];
    diffuse_[1] = color[1];
    diffuse_[2] = color[2];
    diffuse_[3] = color[3];
}

void light::setDiffuse(const float * color)
{
    diffuse_[0] = color[0];
    diffuse_[1] = color[1];
    diffuse_[2] = color[2];
    diffuse_[3] = color[3];
}

void light::setDiffuse(float r, float g, float b, float a)
{
    diffuse_[0] = r;
    diffuse_[1] = g;
    diffuse_[2] = b;
    diffuse_[3] = a;
}

void light::setSpecular(vector4& color)
{
    specular_[0] = color[0];
    specular_[1] = color[1];
    specular_[2] = color[2];
    specular_[3] = color[3];
}

void light::setSpecular(const float * color)
{
    specular_[0] = color[0];
    specular_[1] = color[1];
    specular_[2] = color[2];
    specular_[3] = color[3];
}

void light::setSpecular(float r, float g, float b, float a)
{
    specular_[0] = r;
    specular_[1] = g;
    specular_[2] = b;
    specular_[3] = a;
}

void light::setPosition(vector4& position)
{
    position_[0] = position[0];
    position_[1] = position[1];
    position_[2] = position[2];
    position_[3] = position[3];
}

void light::setPosition(const float * position)
{
    position_[0] = position[0];
    position_[1] = position[1];
    position_[2] = position[2];
    position_[3] = position[3];
}

void light::setPosition(float x, float y, float z, float w)
{
    position_[0] = x;
    position_[1] = y;
    position_[2] = z;
    position_[3] = w;
}

void light::setDirectionalLight(vector3& direction)
{
    setPosition(direction[0], direction[1], direction[2], 0);
}

void light::setDirectionalLight(const float * direction)
{
    setPosition(direction[0], direction[1], direction[2], 0);
}

void light::setDirectionalLight(float x, float y, float z)
{
    setPosition(x, y, z, 0);
}

void light::setPointLight(vector3& position)
{
    setPosition(position[0], position[1], position[1], 1);
}

void light::setPointLight(const float * position)
{
    setPosition(position[0], position[1], position[2], 1);
}

void light::setPointLight(float x, float y, float z)
{
    setPosition(x, y, z, 1);
}

void light::setSpotLightDirection(vector3& direction)
{
    spotDirection_[0] = direction[0];
    spotDirection_[1] = direction[1];
    spotDirection_[2] = direction[2];
}

void light::setSpotLightDirection(const float * direction)
{
    spotDirection_[0] = direction[0];
    spotDirection_[1] = direction[1];
    spotDirection_[2] = direction[2];
}

void light::setSpotLightDirection(float x, float y, float z)
{
    spotDirection_[0] = x;
    spotDirection_[1] = y;
    spotDirection_[2] = z;
}

void light::setSpotLightCutoff(GLfloat cutoff)
{
    spotCutoff_ = cutoff;
}

void light::setSpotLightExponent(GLfloat exponent)
{
    spotExponent_ = exponent;
}

void light::setAttenuation(float constant, float linear, float quadratic)
{
    attenuationConstant_ = constant;
    attenuationLinear_ = linear;
    attenuationQuadratic_ = quadratic;
}

} // namespace
