// light class
// -Kyoung Shin Park (2010 Spring)
//////////////////////////////////////////////////////////////////////
#ifndef __LIGHT_H__
#define __LIGHT_H__

#include <windows.h>
#include <GL/gl.h>
#include "vector3.h"
#include "vector4.h"

namespace oglclass
{

class light
{
public:
    light(void);
    void enable(void);
    void disable(void);

    // set light color (ambient, diffuse, specular)
    void setAmbient(vector4& color);
    void setAmbient(const float * color);
    void setAmbient(float r, float g, float b, float a);
    void setDiffuse(vector4& color);
    void setDiffuse(const float * color);
    void setDiffuse(float r, float g, float b, float a);
    void setSpecular(vector4& color);
    void setSpecular(const float * color);
    void setSpecular(float r, float g, float b, float a);

    // set light position & direction by type (directional, point, spot light)
	void setPosition(vector4& position);
	void setPosition(const float * position);
    void setPosition(float x, float y, float z, float w);
    void setDirectionalLight(vector3& direction);
	void setDirectionalLight(const float * direction);
    void setDirectionalLight(float x, float y, float z);
	void setPointLight(vector3& position);
	void setPointLight(const float * position);
    void setPointLight(float x, float y, float z);
    void setSpotLightDirection(vector3& direction);
    void setSpotLightDirection(const float * direction);
    void setSpotLightDirection(float x, float y, float z);
    void setSpotLightCutoff(float cutoff);
    void setSpotLightExponent(float exponent);
	void setAttenuation(float constant, float linear, float quadratic);

    float* ambient(void) { return ambient_; }
    float* diffuse(void) { return diffuse_; }
    float* specular(void) { return specular_; }
    float* position(void) { return position_; }
    float attenuationConstant(void) { return attenuationConstant_; }
    float attenuationLinear(void) { return attenuationLinear_; }
    float attenuationQuadratic(void) { return attenuationQuadratic_; }
    float spotCutoff(void) { return spotCutoff_; }
    float* spotDirection(void) { return spotDirection_; }
    float spotExponent(void) { return spotExponent_; }

private:
    GLenum lightNumber_;
    float ambient_[4], diffuse_[4], specular_[4];
    float position_[4];
    float attenuationConstant_, attenuationLinear_, attenuationQuadratic_;
    float spotDirection_[3];
    float spotCutoff_, spotExponent_;  
    static int gLightNum_;
};

} // namespace

#endif
