// material class
// -Kyoung Shin Park (2009 Spring)
//////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <GL/gl.h>
#include "material.h"

namespace oglclass
{

material::material(void)
{
    ambient_[0] = 0.2f; ambient_[1] = 0.2f; ambient_[2] = 0.2f; ambient_[3] = 1.0f;
    diffuse_[0] = 0.8f; diffuse_[1] = 0.8f; diffuse_[2] = 0.8f; diffuse_[3] = 1.0f;
    specular_[0] = 0.0f; specular_[1] = 0.0f; specular_[2] = 0.0f; specular_[3] = 1.0f;
    emission_[0] = 0.0f; emission_[1] = 0.0f; emission_[2] = 0.0f; emission_[3] = 1.0f;
    shininess_ = 0.0f;
}

material::material(const color& ambientAndDiffuse)
{
    ambient_[0] = ambientAndDiffuse[0];	ambient_[1] = ambientAndDiffuse[1]; ambient_[2] = ambientAndDiffuse[2]; ambient_[3] = ambientAndDiffuse[3];
    diffuse_[0] = ambientAndDiffuse[0]; diffuse_[1] = ambientAndDiffuse[1]; diffuse_[2] = ambientAndDiffuse[2]; diffuse_[3] = ambientAndDiffuse[3];
    specular_[0] = 0.0f; specular_[1] = 0.0f; specular_[2] = 0.0f; specular_[3] = 1.0f;
    emission_[0] = 0.0f; emission_[1] = 0.0f; emission_[2] = 0.0f; emission_[3] = 1.0f;
    shininess_ = 0.0f;
}

material::material(const color& ambientAndDiffuse, const color& specular, float shininess)
{
    ambient_[0] = ambientAndDiffuse[0]; ambient_[1] = ambientAndDiffuse[1]; ambient_[2] = ambientAndDiffuse[2]; ambient_[3] = ambientAndDiffuse[3];
    diffuse_[0] = ambientAndDiffuse[0]; diffuse_[1] = ambientAndDiffuse[1]; diffuse_[2] = ambientAndDiffuse[2]; diffuse_[3] = ambientAndDiffuse[3];
    specular_[0] = specular[0]; specular_[1] = specular[1]; specular_[2] = specular[2]; specular_[3] = specular[3];
    emission_[0] = 0.0f; emission_[1] = 0.0f; emission_[2] = 0.0f; emission_[3] = 1.0f;
    shininess_ = shininess;
}

void material::enable(void)
{
    glEnable(GL_LIGHTING);
    glMaterialfv(GL_FRONT, GL_AMBIENT, ambient_);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse_);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular_);
    glMaterialfv(GL_FRONT, GL_EMISSION, emission_);
    glMaterialf(GL_FRONT, GL_SHININESS, shininess_);
}

void material::disable(void)
{
    glDisable(GL_LIGHTING);
}

void material::setAmbient(const float * ambient)
{
    ambient_[0] = ambient[0];
    ambient_[1] = ambient[1];
    ambient_[2] = ambient[2];
    ambient_[3] = ambient[3];
}

void material::setAmbient(vector4& ambient)
{
    ambient_[0] = ambient[0];
    ambient_[1] = ambient[1];
    ambient_[2] = ambient[2];
    ambient_[3] = ambient[3];
}

void material::setDiffuse(const float * diffuse)
{
    diffuse_[0] = diffuse[0];
    diffuse_[1] = diffuse[1];
    diffuse_[2] = diffuse[2];
    diffuse_[3] = diffuse[3];
}

void material::setDiffuse(vector4& diffuse)
{
    diffuse_[0] = diffuse[0];
    diffuse_[1] = diffuse[1];
    diffuse_[2] = diffuse[2];
    diffuse_[3] = diffuse[3];
}

void material::setAmbientAndDiffuse(const float * ambientAndDiffuse)
{
    setAmbient(ambientAndDiffuse);
    setDiffuse(ambientAndDiffuse);
}

void material::setAmbientAndDiffuse(vector4& ambientAndDiffuse)
{
    setAmbient(ambientAndDiffuse);
    setDiffuse(ambientAndDiffuse);
}

void material::setSpecular(const float * specular)
{
    specular_[0] = specular[0];
    specular_[1] = specular[1];
    specular_[2] = specular[2];
    specular_[3] = specular[3];
}

void material::setSpecular(vector4& specular)
{
    specular_[0] = specular[0];
    specular_[1] = specular[1];
    specular_[2] = specular[2];
    specular_[3] = specular[3];
}

void material::setEmission(const float * emission)
{
    emission_[0] = emission[0];
    emission_[1] = emission[1];
    emission_[2] = emission[2];
    emission_[3] = emission[3];
}

void material::setEmission(vector4& emission)
{
    emission_[0] = emission[0];
    emission_[1] = emission[1];
    emission_[2] = emission[2];
    emission_[3] = emission[3];
}

void material::setShininess(float shininess)
{
    shininess_ = shininess;
}

} // namespace
