// matrix class
// -Kyoung Shin Park (2010 Spring)
//////////////////////////////////////////////////////////////////////
#ifndef __MATRIX4x4_H__
#define __MATRIX4x4_H__

#include <cmath>
#include "vector3.h"

namespace oglclass {
#define DEGTORAD(degree) ((degree) * (3.141592654f / 180.0f))
#define RADTODEG(radian) ((radian) * (180.0f / 3.141592654f))

class matrix4x4
{
public:
    matrix4x4() { identity(); }
    matrix4x4(float m0, float m4, float  m8, float m12,
              float m1, float m5, float  m9, float m13,
              float m2, float m6, float m10, float m14,
              float m3, float m7, float m11, float m15);
    matrix4x4(const matrix4x4& other);

    // matrix operators...
    void identity(void);
    matrix4x4 transpose(void);
	float matrix4x4::determinant() const;
    matrix4x4 inverse(void);

	// operator overloading
    matrix4x4 operator+ (const matrix4x4 &other);
    matrix4x4 operator- (const matrix4x4 &other);
    matrix4x4 operator* (const matrix4x4 &other);
    matrix4x4 operator* (const float scalar);

	float operator[] (const int i) const;
	float& operator[] (const int i);

	operator float* () const; // type conversion operator overloading

	// matrix transformation
    void translate(vector3 &trans);
    void translate(float x, float y, float z);
    void rotate(const float &angle, vector3 &axis);
    void rotate(const float &angle, unsigned char axis);
    void rotate(const float angle, float x, float y, float z);
    void scale(vector3 &scale);
    void scale(float x, float y, float z);
    void transformPoint(vector3 *vec);
    void transformVector(vector3 *vec);

	friend ostream& operator<<(ostream& s, const matrix4x4& p);

protected:
	float m_[16];

    static const float EPSILON;
	inline static bool closeEnough(float f1, float f2)
    {
        // Determines whether the two floating-point values f1 and f2 are
        // close enough together that they can be considered equal.
        return fabsf((f1 - f2) / ((f2 == 0.0f) ? 1.0f : f2)) < EPSILON;
    }
};

inline std::ostream& operator<< (std::ostream& s, const matrix4x4& m)
{ 
	return s << m.m_[0] << " " << m.m_[4] << " " << m.m_[8] << " " << m.m_[12] << endl
	         << m.m_[1] << " " << m.m_[5] << " " << m.m_[9] << " " << m.m_[13] << endl
	         << m.m_[2] << " " << m.m_[6] << " " << m.m_[10] << " " << m.m_[14] << endl
	         << m.m_[3] << " " << m.m_[7] << " " << m.m_[11] << " " << m.m_[15] << endl;
}

} // namespace

#endif
