// node class
// -Kyoung Shin Park (2010 Spring)
//////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <malloc.h>
#include "node.h"

namespace oglclass
{

node::node(void)
{
	material_ = 0;			// material
	matrixTransform_ = 0;	// matrix transform
	transform_ = 0;			// transform

	parent_ = 0;
	numChildren_ = 0;
	maxChildren_ = 16;
	children_ = (node **) malloc(maxChildren_ * sizeof(node *));

	drawFunc_ = 0;
	drawData_ = 0;
	updateFunc_ = 0;
	updateData_ = 0;
}

node::~node(void)
{
}

void node::drawAll(void)
{
	if (matrixTransform_)
		matrixTransform_->pushMatrix();
	if (transform_)
		transform_->pushMatrix();

	// draw itself
	if (material_)
		material_->enable();
	draw();
	if (material_)
		material_->disable();
	// draw its children
	for (int i=0; i < numChildren_; i++)
		children_[i]->drawAll();

	if (transform_)
		transform_->popMatrix();
	if (matrixTransform_)
		matrixTransform_->popMatrix();
}

void node::updateAll(void)
{
	// update itself
	update();
	// update its children
	for (int i=0; i < numChildren_; i++)
		children_[i]->updateAll();
}

void node::draw(void)
{
	if (drawFunc_)
		(*drawFunc_)(*this,drawData_);
}

void node::update(void)
{
	if (updateFunc_)
		(*updateFunc_)(*this,updateData_);
}

void node::addChild(node& child)
{
	if (numChildren_ == maxChildren_) {
		maxChildren_ *= 2;
		node ** newchildren = (node **)
						realloc(children_, maxChildren_ * sizeof(node *));
		if (newchildren)
			children_ = newchildren;
		else {
			std::cerr << "ERROR: node::attach() failed to realloc space for "
				 << maxChildren_ << " children\n";
			perror("realloc");
			maxChildren_ /= 2;
			return;
			}
	}
	if (child.parent_)
		child.parent_->removeChild(child);
	child.parent_ = this;
	children_[numChildren_] = &child;
	numChildren_++;
}

void node::removeChild(node& child)
{
	int i;
	for (i=0; i < numChildren_; i++) {
		if (children_[i] == &child) {
			child.parent_ = 0;
			for (; i < numChildren_-1; i++)
				children_[i] = children_[i+1];
			numChildren_--;
			break;
		}
	}
}

int node::numChildren(void)
{
	return numChildren_;
}

node& node::getChild(int n)
{
	if ((n < 0) || (n >= numChildren_))
		return *this;
	return *(children_[n]);
}

void node::setDrawCallback(CallbackFunc drawfunc,void *drawdata)
{
	drawFunc_ = drawfunc;
	drawData_ = drawdata;
}

void node::setUpdateCallback(CallbackFunc updatefunc,void *updatedata)
{
	updateFunc_ = updatefunc;
	updateData_ = updatedata;
}

} // namespace
