// plane class
// -Kyoung Shin Park (2011 Spring)
//////////////////////////////////////////////////////////////////////
#include "plane.h"

namespace oglclass
{
const float plane::EPSILON = 1e-6f;

plane::plane (const float *other)
{
    set(other[0], other[1], other[2], other[3]);
}

plane::plane (float a, float b, float c, float d)
{
    set(a, b, c, d);
}

plane::plane (vector3 p0, vector3 p1, vector3 p2)
{
    set(p0, p1, p2);
}

plane::plane (vector3 n, vector3 p)
{
    set(n, p);
}

void plane::set (float a, float b, float c, float d)
{
    v_[0] = a;
    v_[1] = b;
    v_[2] = c;
    v_[3] = d;
    normalize();
}

void plane::set (vector3 p0, vector3 p1, vector3 p2)
{
	vector3 u, v, n;
	u = p2 - p0;
	v = p1 - p0;
    n = vector3::crossProduct(u, v);
	n.normalize();
	v_[0] = n[0];
	v_[1] = n[1];
	v_[2] = n[2];
	v_[3] = -vector3::dotProduct(n, p0);
}

void plane::set (vector3 n, vector3 p)
{
	v_[0] = n[0];
	v_[1] = n[1];
	v_[2] = n[2];
	v_[3] = -vector3::dotProduct(n, p);
}

float plane::length (void)
{
    return ( (float)sqrt( v_[0] * v_[0] + v_[1] * v_[1] + v_[2] * v_[2] + v_[3] * v_[3]) );
}

void plane::normalize (void)
{
    float fLength = length();

    v_[0] = v_[0] / fLength;
    v_[1] = v_[1] / fLength;
    v_[2] = v_[2] / fLength;
    v_[3] = v_[3] / fLength;
}

// a*x + b*y + c*z + d*0
// n . v (where n = [a, b, c])
float plane::dotNormal (const plane &p, const vector3 &v)
{
    return (p[0] * v[0] + p[1] * v[1] + p[2] * v[2]);
}

// a*x + b*y + c*z + d*1
// n . v + d (where n = [a, b, c])
float plane::dotCoord (const plane &p, const vector3 &v)
{
	/*float a, b, c, d, x, y, z;
	a = p[0]; b = p[1]; c = p[2]; d = p[3];
	x = v[0]; y = v[1]; z = v[2];
    return (a * x + b * y + c * z + d);*/
    return (p[0] * v[0] + p[1] * v[1] + p[2] * v[2] + p[3]);
}

// the closest point equation: result = v - (n * v + d) * n (where n = [a, b, c])
// dotCoord is the closest distance from point to plane
vector3 plane::closestPointOnPlane (const plane &p, const vector3 &v)
{
    float result[3];

    result[0] = v[0] - dotCoord(p, v) * p[0];
    result[1] = v[1] - dotCoord(p, v) * p[1];
    result[2] = v[2] - dotCoord(p, v) * p[2];

    return vector3(result[0], result[1], result[2]);
}

// check if the point is inside/outside/on the plane
int plane::isPointInsideOutside(const plane &p, const vector3 &v)	
{
	float det = dotCoord(p, v);
	printf("isPointInsideOutside det=%f\n", det);
	if (plane::closeEnough(det, 0.0f))
		return ON_PLANE;		// on the plane
	else if (det > 0.0f)
		return OUTSIDE_PLANE;	// outside
	else //if (det < 0.0f)
		return INSIDE_PLANE;	// inside
}
/*
// equality operators
bool plane::operator == (plane &other) const
{
	return (v_[0] == other[0] && v_[1] == other[1] && v_[2] == other[2] && v_[3] == other[3]);
}

bool plane::operator != (plane &other) const
{
	return (v_[0] != other[0] && v_[1] != other[1] && v_[2] != other[2] && v_[3] != other[3]);
}

// operator[] overloading
float plane::operator [] (const int i) const
{
	if (i == 0)			// if i==0, return a
		return v_[0];
	else if (i == 1)	// else if i==1, return b
		return v_[1];
	else if (i == 2)	// else if i==2, return c
		return v_[2];
	return v_[3];			// otherwise, return d
}

float& plane::operator [] (const int i)
{
	if (i == 0)			// if i==0, return a
		return v_[0];
	else if (i == 1)	// else if i==1, return b
		return v_[1];
	else if (i == 2)	// else if i==2, return c
		return v_[2];
	return v_[3];			// otherwise, return d
}
*/
} // namespace