// plane class
// -Kyoung Shin Park (2010 Spring)
//////////////////////////////////////////////////////////////////////
#ifndef _PLANE_H_
#define _PLANE_H_

#include <cmath>
#include <iostream>
#include "vector3.h"
#include "vector4.h"
using namespace std;

namespace oglclass
{
#define ON_PLANE 0
#define OUTSIDE_PLANE 1
#define INSIDE_PLANE -1

class plane : public vector4
{
public:

	// constructor
	plane(void) : vector4() {}
    plane(const float *other);
    plane(float a, float b, float c, float d);
    plane(vector3 p0, vector3 p1, vector3 p2);
    plane(vector3 n, vector3 p);

	void set(float a, float b, float c, float d);	// make a plane with a, b, c, d
    void set(vector3 p0, vector3 p1, vector3 p2);	// make a plane with 3 points
    void set(vector3 n, vector3 p);					// make a plane with a normal vector and a point on a plane

	// length & normalize
    float length(void);
    void normalize(void);

	// static utility methods (plane p: a, b, c, d & vector3 v: x, y, z)
    static float dotNormal(const plane &p, const vector3 &v);   // a*x + b*y + c*z + d*0
    static float dotCoord(const plane &p, const vector3 &v);    // a*x + b*y + c*z + d*1
    static vector3 closestPointOnPlane(const plane &p, const vector3 &v);
	static int isPointInsideOutside(const plane &p, const vector3 &v);	// return INSIDE(-1)/OUTSIDE(1)/ON_THE_PLANE(0)

    // assignment operators...
    //plane& operator = (plane &other);

    // equality operators
	//bool operator == (plane &other) const;
	//bool operator != (plane &other) const;

	//float operator[] (const int i) const;
	//float& operator[] (const int i);
	//operator float* () const;						// type conversion operator overloading

#ifdef OPENGL
    void draw();
#endif

	friend ostream& operator<<(ostream& s, const plane& p);

protected:
    //float a_, b_, c_, d_;

    static const float EPSILON;
	inline static bool closeEnough(float f1, float f2)
    {
        // Determines whether the two floating-point values f1 and f2 are
        // close enough together that they can be considered equal.
        return fabsf((f1 - f2) / ((f2 == 0.0f) ? 1.0f : f2)) < EPSILON;
    }
};

inline ostream& operator<< (ostream& s, const plane& p)
{ 
	return s << "PLANE (" << p.v_[0] << ", " << p.v_[1] << ", " << p.v_[2] << ", " << p.v_[3] << ")"; 
}

} // namespace

#endif
