
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "vector3.h"
#include "vector4.h"
#include "matrix4x4.h"
#include "plane.h"

#include "matrixtransform.h"
#include "transform.h"

#include "color.h"
#include "material.h"
#include "light.h"

#include "geometry.h"
#include "cube.h"
#include "weed.h"

#include "camera.h"

using namespace oglclass;

// update
float  g_ElapsedTime = 0.;
double g_CurrentTime = 0.;
double g_PreviousTime = 0.;
double g_DeltaTime = 0.;
float g_Speed = 1.0f;
bool  g_OrbitOn = false;

// sun, earth, moon
float g_SunRadius = 5.0f;
float g_EarthRadius = 1.0f;
float g_MoonRadius = 0.5f;
float g_EarthDistanceFromSun = -12.0f;
float g_MoonDistanceFromEarth = -2.0f;

camera camera1(45, 1, 0.1f, 100.0f);

light light0;
light light1;

matrixtransform * sunMatrix;
matrixtransform * earthMatrix;
matrixtransform * moonMatrix;
geometry sun;
geometry earth;
geometry moon;

geometry ground;
geometry lake;

weed bush;
weed grass;

// root node
node root;

void initWeeds(node & root)
{
	float pos[4];

	//ȭ鿡 ׷ ġ Ƿ 200 .
	for(int i = 0; i < 200; i++)
	{
		weed *tWeed = new weed();
		float posX, posZ;
		posX = (rand() % 20) + 10;
		posZ = (rand() % 10) + 15;
		if ((rand() % 2))
			posX *= -1;
		if ((rand() % 2))
			posZ *= -1;

		pos[0] = posX;	pos[1] = -0.1;
		pos[2] = posZ;	pos[3] = rand() % 6 + 6;
		weedType type = (weedType) (rand() % 3);
		transform *tr = new transform;
		if (type == WEED_GRASS) {
			tr->setTranslation(vector3(pos[0], pos[1], pos[2]));//, pos[3]);
		}
		else if (type == WEED_BUSH) {
			int t2 = rand() % 5;
			tr->setTranslation(vector3(pos[0] + t2, pos[1], pos[2] + t2));//, pos[3] + 3);
		}
		else {
			int t2 = rand() % 5;
			tr->setTranslation(vector3(pos[0] - t2, pos[1], pos[2] - t2));//, pos[3] + 3);
		}
		tWeed->setTransform(tr);
		tWeed->setType(type);
		tWeed->setMaterial(new material(color::DarkGreen));
		root.addChild(*tWeed);
	}
}

void createScene(node & root)
{
	// 
	material * groundMaterial = new material(color::Orange);
	transform * t1 = new transform();
	t1->setRotation(90, vector3(1.0f, 0.0f, 0.0f));
	ground.makeDisk(0, 50, 32);
	ground.setMaterial(groundMaterial);
	ground.setTransform(t1);

	// ȣ
	transform * t2 = new transform();
	t2->setTranslation(vector3(15.0, 0.01, 15.0)); 
	t2->setRotation(90, vector3(1.0f, 0.0f, 0.0f));
	lake.makeDisk(0, 10, 30);
	lake.setMaterial(new material(color::SkyBlue));
	lake.setTransform(t2);

	// 
	bush.setType(WEED_BUSH);
	bush.setMaterial(new material(color::DarkGreen));

	transform * t = new transform();
	t->setTranslation(0, 5, 0);
	t->setScaling(5, 5, 5);
	grass.setTransform(t);
	grass.setMaterial(new material(color::YellowGreen));

	// ¾, , 
	sun.makeSphere(g_SunRadius, 32, 24);
	material * sunMaterial = new material(color::Yellow);
	sun.setMaterial(sunMaterial);

	earth.makeSphere(g_EarthRadius, 32, 24);
	material * earthMaterial = new material(color::Green);
	earth.setMaterial(earthMaterial);

	moon.makeSphere(g_MoonRadius, 32, 24);
	material * moonMaterial = new material(color::White);
	moon.setMaterial(moonMaterial);

	sunMatrix = new matrixtransform;
	earthMatrix = new matrixtransform;
	moonMatrix = new matrixtransform;

	// scene graph
	root.addChild(ground);
	root.addChild(lake);
	root.addChild(sun);
	root.addChild(earth);
	earth.addChild(moon);

	earth.addChild(bush);
	sun.addChild(grass);

	initWeeds(root);
}

void init(void)
{
    glClearColor( 0.1f, 0.4f, 1.0f, 1.0f );
    glEnable(GL_DEPTH_TEST);

	// ambient directional light
	light0.setDirectionalLight(1, 1, 0);
	light0.setDiffuse(0.9, 0.9, 0.9, 1);
	light0.enable();

	light1.setDirectionalLight(0, 1, -1);
	light1.setDiffuse(1, 1, 1, 1);
	light1.enable();

	camera1.applyProjection();
	camera1.setPosition(0.0, 2.0, 25.0);

	createScene(root);
}

void keyboard(unsigned char key, int x, int y)
{
	switch(key) {
		case 27:
			exit(0);
			break;
		case ' ':
			g_OrbitOn = !g_OrbitOn;
			break;
		case '+':
			++g_Speed;
			break;
		case '-':
			--g_Speed;
			break;
		default:
			break;
	}
}

void specialkey(int key, int x, int y)
{
	if (key == GLUT_KEY_F1)			// x-movement
        camera1.strafe(0.5);
	else if (key == GLUT_KEY_F2)
        camera1.strafe(-0.5);
	else if (key == GLUT_KEY_F3)	// y-movement
        camera1.fly(0.5);
	else if (key == GLUT_KEY_F4)
        camera1.fly(-0.5);
	else if (key == GLUT_KEY_F5)	// z-movement
        camera1.walk(0.5);
    else if (key == GLUT_KEY_F6)
        camera1.walk(-0.5);

	else if (key == GLUT_KEY_F7)	// yaw (by y-axis)
        camera1.yaw(2.5);
	else if (key == GLUT_KEY_F8)
        camera1.yaw(-2.5);
	else if (key == GLUT_KEY_F9)	// pitch (by x-axis)
        camera1.pitch(2.5);
    else if (key == GLUT_KEY_F10)
        camera1.pitch(-2.5);
	else if (key == GLUT_KEY_HOME)	// roll (by z-axis)
        camera1.roll(2.5);
    else if (key == GLUT_KEY_END)
        camera1.roll(-2.5);

	else if (key == GLUT_KEY_LEFT)
        camera1.yaw(2.5);			// pan right
	else if (key == GLUT_KEY_RIGHT)
        camera1.yaw(-2.5);			// pan left
	else if (key == GLUT_KEY_UP)	
        camera1.walk(0.5);			// move forward
    else if (key == GLUT_KEY_DOWN)
        camera1.walk(-0.5);			// move backward

	glutPostRedisplay();
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	camera1.setAspect((float)w / (float)h);
}

void display( void )
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	camera1.apply();
	root.drawAll();

	glutSwapBuffers();
}

void update()
{
	g_CurrentTime = timeGetTime();
	g_DeltaTime = g_CurrentTime - g_PreviousTime;
	g_ElapsedTime = (float)(g_DeltaTime * 0.001);
	g_PreviousTime = g_CurrentTime;

	static float SunSpin    = 0.0f;    
    static float EarthSpin  = 0.0f;    
    static float EarthOrbit = 0.0f;
    static float MoonSpin   = 0.0f;
    static float MoonOrbit  = 0.0f;

	if( g_OrbitOn == true ) {
        SunSpin -= g_Speed * (g_ElapsedTime * 10.0f);		// sun rotate

        EarthSpin  -= g_Speed * (g_ElapsedTime * 100.0f);	// earth rotate
        EarthOrbit -= g_Speed * (g_ElapsedTime * 20.0f);	// earth revolve

        MoonSpin  -= g_Speed * (g_ElapsedTime * 50.0f);		// moon rotate
        MoonOrbit -= g_Speed * (g_ElapsedTime * 200.0f);	// moon revolve
    }   

	sunMatrix->rotate(SunSpin, 0, 1, 0);
	sun.setMatrixTransform(sunMatrix);

	matrix4x4 t, r1, r2, m;
	t.translate(0, 0, g_EarthDistanceFromSun);
	r1.rotate(EarthSpin, 0, 1, 0);
	r2.rotate(EarthOrbit, 0, 1, 0);
	m = r2 * t * r1;
	earthMatrix->setMatrix(m);
	earth.setMatrixTransform(earthMatrix);

	t.translate(0, 0, g_MoonDistanceFromEarth);
	r1.rotate(MoonSpin, 0, 1, 0);
	r2.rotate(MoonOrbit, 0, 1, 0);
	m = r2 * t * r1;
	moonMatrix->setMatrix(m);
	moon.setMatrixTransform(moonMatrix);

	glutPostRedisplay();
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);
	glutInitWindowSize(500, 500);
	glutInitWindowPosition(100, 100);
	glutCreateWindow(argv[0]);
	init();
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialkey);
	glutIdleFunc(update);
	glutMainLoop();
	return 0;
}
