// vector3 class
// -Kyoung Shin Park (2011 Spring)
//////////////////////////////////////////////////////////////////////
#include <cmath>
#include "vector3.h"

namespace oglclass
{

vector3 vector3::Zero(0.0, 0.0, 0.0);

vector3::vector3(const vector3 &other)
{
    v_[0] = other.v_[0];
    v_[1] = other.v_[1];
    v_[2] = other.v_[2];
}

vector3::vector3(const float *other)
{
    v_[0] = other[0];
    v_[1] = other[1];
    v_[2] = other[2];
}

vector3::vector3(float x, float y, float z)
{
    v_[0] = x;
    v_[1] = y;
    v_[2] = z;
}

void vector3::set(float x, float y, float z)
{
    v_[0] = x;
    v_[1] = y;
    v_[2] = z;
}

float vector3::length(void)
{
    return( (float)sqrt( v_[0] * v_[0] + v_[1] * v_[1] + v_[2] * v_[2] ) );
}

void vector3::normalize(void)
{
    float fLength = length();

    v_[0] = v_[0] / fLength;
    v_[1] = v_[1] / fLength;
    v_[2] = v_[2] / fLength;
}

float vector3::distance(const vector3 &v1, const vector3 &v2)
{
    float dx = v1.v_[0] - v2.v_[0];
    float dy = v1.v_[1] - v2.v_[1];
    float dz = v1.v_[2] - v2.v_[2];

    return (float)sqrt( dx * dx + dy * dy + dz * dz );
}

float vector3::dotProduct(const vector3 &v1, const vector3 &v2)
{
    return (v1.v_[0] * v2.v_[0] + v1.v_[1] * v2.v_[1] + v1.v_[2] * v2.v_[2]);
}

vector3 vector3::crossProduct(const vector3 &v1, const vector3 &v2)
{
	vector3 result;

    result.v_[0] = v1.v_[1] * v2.v_[2] - v1.v_[2] * v2.v_[1];
    result.v_[1] = v1.v_[2] * v2.v_[0] - v1.v_[0] * v2.v_[2];
    result.v_[2] = v1.v_[0] * v2.v_[1] - v1.v_[1] * v2.v_[0];

    return result;
}

float vector3::theta(const vector3 &v1,  const vector3 &v2)
{
	float len1 = (float)sqrt(v1.v_[0]*v1.v_[0] + v1.v_[1]*v1.v_[1] + v1.v_[2]*v1.v_[2]);
	float len2 = (float)sqrt(v2.v_[0]*v2.v_[0] + v2.v_[1]*v2.v_[1] + v2.v_[2]*v2.v_[2]);
	float dot = dotProduct(v1, v2);
	float result = (float)acos(dot/len1*len2);
    return result;
}

// operators...
vector3& vector3::operator = (const vector3 &other)
{
    v_[0] = other.v_[0];
    v_[1] = other.v_[1];
    v_[2] = other.v_[2];

    return *this;
}

vector3& vector3::operator += (const vector3 &other)
{
    v_[0] += other.v_[0];
    v_[1] += other.v_[1];
    v_[2] += other.v_[2];

    return *this;
}

vector3& vector3::operator -= (const vector3 &other)
{
    v_[0] -= other.v_[0];
    v_[1] -= other.v_[1];
    v_[2] -= other.v_[2];

    return *this;
}

 vector3& vector3::operator *= (const float scalar)
{
    v_[0] *= scalar;
    v_[1] *= scalar;
    v_[2] *= scalar;

    return *this;
}

vector3& vector3::operator /= (const float scalar)
{
    v_[0] /= scalar;
    v_[1] /= scalar;
    v_[2] /= scalar;

    return *this;
}

bool vector3::operator == (const vector3 &other) const
{
	return (v_[0] == other.v_[0] && v_[1] == other.v_[1] && v_[2] == other.v_[2]);
}

bool vector3::operator != (const vector3 &other) const
{
	return (v_[0] != other.v_[0] || v_[1] != other.v_[1] || v_[2] != other.v_[2]);
}

vector3 vector3::operator + (const vector3 &other)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] + other.v_[0];
    result.v_[1] = v_[1] + other.v_[1];
    result.v_[2] = v_[2] + other.v_[2];

    return result;
}

vector3 vector3::operator - (const vector3 &other)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] - other.v_[0];
    result.v_[1] = v_[1] - other.v_[1];
    result.v_[2] = v_[2] - other.v_[2];

    return result;
}

vector3 vector3::operator * (const vector3 &other)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] * other.v_[0];
    result.v_[1] = v_[1] * other.v_[1];
    result.v_[2] = v_[2] * other.v_[2];

    return result;
}

vector3 vector3::operator / (const vector3 &other)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] / other.v_[0];
    result.v_[1] = v_[1] / other.v_[1];
    result.v_[2] = v_[2] / other.v_[2];

    return result;
}

vector3 vector3::operator * (const float scalar)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] * scalar;
    result.v_[1] = v_[1] * scalar;
    result.v_[2] = v_[2] * scalar;

    return result;
}

vector3 vector3::operator / (const float scalar)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = v_[0] / scalar;
    result.v_[1] = v_[1] / scalar;
    result.v_[2] = v_[2] / scalar;

    return result;
}

vector3 operator * (const float scalar, const vector3 &other)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = other.v_[0] * scalar;
    result.v_[1] = other.v_[1] * scalar;
    result.v_[2] = other.v_[2] * scalar;

    return result;
}

vector3 operator / (const float scalar, const vector3 &other)
{
    vector3 result(0.0f, 0.0f, 0.0f);

    result.v_[0] = other.v_[0] / scalar;
    result.v_[1] = other.v_[1] / scalar;
    result.v_[2] = other.v_[2] / scalar;

    return result;
}

vector3 vector3::operator + (void) const
{
    return *this;
}

vector3 vector3::operator - (void) const
{
    vector3 result(-v_[0], -v_[1], -v_[2]);

    return result;
}

float vector3::operator [] (const int i) const
{
	if (i == 0)			// if i==0, return x
		return v_[0];
	else if (i == 1)	// else if i==1, return y
		return v_[1];
	return v_[2];			// otherwise, return z
}

float& vector3::operator [] (const int i)
{
	if (i == 0)			// if i==0, return x
		return v_[0];
	else if (i == 1)	// else if i==1, return y
		return v_[1];
	return v_[2];			// otherwise, return z
}

} // namespace